import { type NestedStringArray } from "@askvortsov/rich-icu-message-formatter";
import app from "flarum/admin/app";
import type Model from "flarum/common/Model";
import Button from "flarum/common/components/Button";
import Modal from "flarum/common/components/Modal";
import Stream from "flarum/common/utils/Stream";

export default abstract class EditModelModal<T extends Model> extends Modal {
    model: T | undefined;

    saving = false;
    fieldValues: { [x: string]: Stream } = {};

    oninit(vnode: any): void {
        super.oninit(vnode);

        this.model = (this.attrs as any).model;

        for (const field of this.fields()) {
            this.fieldValues[field] = Stream(this.model ? ((this.model as any)[field]() ?? "") : "");
        }

        if (!this.model) {
            const defaults = this.defaultValues();
            for (const field in defaults) {
                this.fieldValues[field](defaults[field]);
            }
        }
    }

    abstract modelType(): string;

    abstract fields(): string[];

    abstract defaultValues(): { [x: string]: string }

    abstract translateFieldName(field: string): NestedStringArray

    className(): string {
        return "OAuth-EditModel";
    }
    content() {
        return (
            <div className="Modal-body">
                {this.fields().map(field => <div className="Form-group">
                    <label for={field}>{this.translateFieldName(field)}</label>
                    <input
                        className="FormControl"
                        name={field}
                        type="text"
                        bidi={this.fieldValues[field]}
                        disabled={this.saving}
                        />
                </div>)}
                <Button className="Button Button--primary Button--block" onclick={this.save.bind(this)} disabled={this.saving} loading={this.saving}>{app.translator.trans("rhodes-island-oauth-center.admin.clients.edit_modal_save")}</Button>
            </div>
        );
    }

    save() {
        this.saving = true;
        if (!this.model) {
            this.model = app.store.createRecord(this.modelType()) as T;
        }
        let modifiedCount = 0;
        const attrs: { [x: string]: string } = {};
        Object.entries(this.fieldValues).forEach(val => {
            const value = val[1]();
            const modelValue = (this.model as any)[val[0]]();
            if (value != modelValue && (value != "" || modelValue != null)) {
                attrs[val[0]] = value;
                modifiedCount++;
            }
        });
        if (modifiedCount == 0) {
            this.saving = false;
            return;
        };
        this.model.save(attrs).finally(() => {
            this.saving = false;
            if (typeof (this.attrs as any).createCallback === "function") {
                (this.attrs as any).createCallback();
            }
            m.redraw();
        });
    }
}