import app from 'flarum/admin/app';
import Page from 'flarum/common/components/Page';
import Button from 'flarum/common/components/Button';
import LoadingIndicator from "flarum/common/components/LoadingIndicator";
import type Client from 'src/common/models/Client';
import EditApplicationModal from '../components/EditApplicationModal';

export default class ClientsPage extends Page {
  translationPrefix = 'rhodes-island-oauth-center.admin.clients.';
  loading = false;
  clients: Client[] = [];
  tableFields = [
    "client_id",
    "client_name",
    "client_desc"
  ];
  loadingClients: Client[] = [];

  oninit(vnode: any) {
    super.oninit(vnode);

    this.refresh();
  }

  refresh() {
    this.loading = true;
    app.store.find('oauth-clients').then(r => {
      this.clients = r as unknown as Client[];
      this.loading = false;
      m.redraw();
    });
  }

  view() {
    return (
      <div class="OAuthCenter-clientsPage">
        {(this.loading ? <LoadingIndicator></LoadingIndicator> : <div className="Form-group">
          <table>
            <thead>
              <tr>
                {this.tableFields.map(key => <th>{app.translator.trans(this.translationPrefix + key)}</th>)}
                <th>{app.translator.trans(this.translationPrefix + "actions")}</th>
              </tr>
            </thead>
            <tbody>
              {this.clients.map((client: any) => {
                const clientSaving = this.loadingClients.indexOf(client) != -1;
                return <tr>
                  {this.tableFields.map(key => <td>{client[key]()}</td>)}
                  <td>
                    <Button className="Button Button--icon" icon="fas fa-pen" disabled={clientSaving} onclick={() => {
                      app.modal.show(EditApplicationModal, {
                        model: client
                      });
                    }}></Button>
                    &nbsp;
                    <Button className="Button Button--icon" icon="fas fa-times" disabled={clientSaving} onclick={() => {
                      this.loadingClients.push(client);
                      (client as Client).delete()
                        .then(() => this.clients.splice(this.clients.indexOf(client), 1))
                        .finally(() => this.loadingClients.splice(this.loadingClients.indexOf(client), 1))
                        .finally(m.redraw);
                    }}></Button>
                  </td>
                </tr>
                }
              )}
            </tbody>
          </table>
          <Button className="Button Button--block" onclick={() => {
            app.modal.show(EditApplicationModal, {
              createCallback: this.refresh.bind(this)
            });
          }}>{app.translator.trans(this.translationPrefix + "add_button")}</Button>
        </div>)}
      </div>
    );
  }
}
